@@ -924,6 +924,13 @@ chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem,
 				  chat_spell->word);
 }
 
+static void
+chat_text_send_cb (GtkMenuItem *menuitem,
+		   EmpathyChat *chat)
+{
+	chat_input_text_view_send (chat);
+}
+
 static void
 chat_input_populate_popup_cb (GtkTextView *view,
 			      GtkMenu     *menu,
@@ -939,15 +946,20 @@ chat_input_populate_popup_cb (GtkTextView *view,
 	gchar           *str = NULL;
 	EmpathyChatSpell *chat_spell;
 	GtkWidget       *smiley_menu;
+	GtkWidget       *image;
 
 	priv = GET_PRIV (chat);
+	buffer = gtk_text_view_get_buffer (view);
 
 	/* Add the emoticon menu. */
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
-	item = gtk_menu_item_new_with_mnemonic (_("Insert Smiley"));
+	item = gtk_image_menu_item_new_with_mnemonic (_("Insert Smiley"));
+	image = gtk_image_new_from_icon_name ("face-smile",
+					      GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
 
@@ -956,51 +968,55 @@ chat_input_populate_popup_cb (GtkTextView *view,
 		chat);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), smiley_menu);
 
+	/* Add the Send menu item. */
+	gtk_text_buffer_get_bounds (buffer, &start, &end);
+	str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+	if (!G_STR_EMPTY (str)) {
+		item = gtk_menu_item_new_with_mnemonic (_("_Send"));
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (chat_text_send_cb), chat);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
+	}
+	str = NULL;
+
 	/* Add the spell check menu item. */
-	buffer = gtk_text_view_get_buffer (view);
 	table = gtk_text_buffer_get_tag_table (buffer);
-
 	tag = gtk_text_tag_table_lookup (table, "misspelled");
-
 	gtk_widget_get_pointer (GTK_WIDGET (view), &x, &y);
-
 	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (view),
 					       GTK_TEXT_WINDOW_WIDGET,
 					       x, y,
 					       &x, &y);
-
 	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (view), &iter, x, y);
-
 	start = end = iter;
-
 	if (gtk_text_iter_backward_to_tag_toggle (&start, tag) &&
 	    gtk_text_iter_forward_to_tag_toggle (&end, tag)) {
 
 		str = gtk_text_buffer_get_text (buffer,
 						&start, &end, FALSE);
 	}
+	if (!G_STR_EMPTY (str)) {
+		chat_spell = chat_spell_new (chat, str, start, end);
+		g_object_set_data_full (G_OBJECT (menu),
+					"chat_spell", chat_spell,
+					(GDestroyNotify) chat_spell_free);
 
-	if (G_STR_EMPTY (str)) {
-		return;
+		item = gtk_separator_menu_item_new ();
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
+
+		item = gtk_image_menu_item_new_with_mnemonic (_("_Check Word Spelling..."));
+		image = gtk_image_new_from_icon_name (GTK_STOCK_SPELL_CHECK,
+						      GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_signal_connect (item,
+				  "activate",
+				  G_CALLBACK (chat_text_check_word_spelling_cb),
+				  chat_spell);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
 	}
-
-	chat_spell = chat_spell_new (chat, str, start, end);
-
-	g_object_set_data_full (G_OBJECT (menu),
-				"chat_spell", chat_spell,
-				(GDestroyNotify) chat_spell_free);
-
-	item = gtk_separator_menu_item_new ();
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	item = gtk_menu_item_new_with_mnemonic (_("_Check Word Spelling..."));
-	g_signal_connect (item,
-			  "activate",
-			  G_CALLBACK (chat_text_check_word_spelling_cb),
-			  chat_spell);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
 }
 
 static void
