@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 
@@ -36,6 +37,7 @@
 #include <libempathy/empathy-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-groups.h>
+#include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-contact-list-view.h"
@@ -51,9 +53,6 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include <libempathy/empathy-debug.h>
 
-/* Flashing delay for icons (milliseconds). */
-#define FLASH_TIMEOUT 500
-
 /* Active users are those which have recently changed state
  * (e.g. online, offline or from normal to a busy state).
  */
@@ -64,6 +63,7 @@ typedef struct {
 	GtkTreeRowReference            *drag_row;
 	EmpathyContactListFeatureFlags  list_features;
 	EmpathyContactFeatureFlags      contact_features;
+	GtkWidget                      *tooltip_widget;
 } EmpathyContactListViewPriv;
 
 typedef struct {
@@ -114,6 +114,58 @@ static guint signals[LAST_SIGNAL];
 
 G_DEFINE_TYPE (EmpathyContactListView, empathy_contact_list_view, GTK_TYPE_TREE_VIEW);
 
+static gboolean
+contact_list_view_query_tooltip_cb (EmpathyContactListView *view,
+				    gint                    x,
+				    gint                    y,
+				    gboolean                keyboard_mode,
+				    GtkTooltip             *tooltip,
+				    gpointer                user_data)
+{
+	EmpathyContactListViewPriv *priv = GET_PRIV (view);
+	EmpathyContact             *contact;
+	GtkTreeModel               *model;
+	GtkTreeIter                 iter;
+	GtkTreePath                *path;
+
+	/* FIXME: We need GTK version >= 2.12.10. See GNOME bug #504087 */
+	if (gtk_check_version (2, 12, 10)) {
+		return FALSE;
+	}
+
+	if (!gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW (view), &x, &y,
+						keyboard_mode,
+						&model, &path, &iter)) {
+		return FALSE;
+	}
+
+	gtk_tree_view_set_tooltip_row (GTK_TREE_VIEW (view), tooltip, path);
+	gtk_tree_path_free (path);
+
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
+	if (!contact) {
+		return FALSE;
+	}
+
+	if (!priv->tooltip_widget) {
+		priv->tooltip_widget = empathy_contact_widget_new (contact,
+			EMPATHY_CONTACT_WIDGET_EDIT_NONE);
+		g_object_add_weak_pointer (G_OBJECT (priv->tooltip_widget),
+					   (gpointer) &priv->tooltip_widget);
+	} else {
+		empathy_contact_widget_set_contact (priv->tooltip_widget,
+						    contact);
+	}
+
+	gtk_tooltip_set_custom (tooltip, priv->tooltip_widget);
+
+	g_object_unref (contact);
+
+	return TRUE;
+}
+
 static void
 contact_list_view_drag_data_received (GtkWidget         *widget,
 				      GdkDragContext    *context,
@@ -405,17 +457,13 @@ contact_list_view_popup_menu_idle_cb (gpointer user_data)
 		menu = empathy_contact_list_view_get_group_menu (data->view);
 	}
 
-	if (!menu) {
-		goto OUT;
+	if (menu) {
+		gtk_widget_show (menu);
+		gtk_menu_popup (GTK_MENU (menu),
+				NULL, NULL, NULL, NULL,
+				data->button, data->time);
 	}
 
-	gtk_widget_show (menu);
-
-	gtk_menu_popup (GTK_MENU (menu),
-			NULL, NULL, NULL, NULL,
-			data->button, data->time);
-
-OUT:
 	g_slice_free (MenuPopupData, data);
 
 	return FALSE;
@@ -439,11 +487,28 @@ contact_list_view_button_press_event_cb (EmpathyContactListView *view,
 	return FALSE;
 }
 
+static gboolean
+contact_list_view_key_press_event_cb (EmpathyContactListView *view,
+				      GdkEventKey	     *event,
+				      gpointer		      user_data)
+{
+	if (event->keyval == GDK_Menu) {
+		MenuPopupData *data;
+
+		data = g_slice_new (MenuPopupData);
+		data->view = view;
+		data->button = 0;
+		data->time = event->time;
+		g_idle_add (contact_list_view_popup_menu_idle_cb, data);
+	}
+
+	return FALSE;
+}
+
 static void
-contact_list_view_row_activated_cb (EmpathyContactListView *view,
-				    GtkTreePath            *path,
-				    GtkTreeViewColumn      *col,
-				    gpointer                user_data)
+contact_list_view_row_activated (GtkTreeView       *view,
+				 GtkTreePath       *path,
+				 GtkTreeViewColumn *column)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
 	EmpathyContact             *contact;
@@ -454,15 +519,15 @@ contact_list_view_row_activated_cb (EmpathyContactListView *view,
 		return;
 	}
 
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-
+	model = GTK_TREE_MODEL (priv->store);
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get (model, &iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
 			    -1);
 
 	if (contact) {
-		empathy_chat_with_contact (contact);
+		DEBUG ("Starting a chat");
+		empathy_dispatcher_chat_with_contact (contact);
 		g_object_unref (contact);
 	}
 }
@@ -491,7 +556,7 @@ contact_list_view_voip_activated_cb (EmpathyCellRendererActivatable *cell,
 			    -1);
 
 	if (contact) {
-		empathy_call_with_contact (contact);
+		empathy_dispatcher_call_with_contact (contact);
 		g_object_unref (contact);
 	}
 }
@@ -857,6 +922,7 @@ contact_list_view_set_list_features (EmpathyContactListView         *view,
 				     EmpathyContactListFeatureFlags  features)
 {
 	EmpathyContactListViewPriv *priv = GET_PRIV (view);
+	gboolean                    has_tooltip;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_LIST_VIEW (view));
 
@@ -881,9 +947,13 @@ contact_list_view_set_list_features (EmpathyContactListView         *view,
 				   G_N_ELEMENTS (drag_types_dest),
 				   GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	} else {
-		/* FIXME: URI could still be  droped depending on FT feature */
+		/* FIXME: URI could still be droped depending on FT feature */
 		gtk_drag_dest_unset (GTK_WIDGET (view));
 	}
+
+	/* Update has-tooltip */
+	has_tooltip = (features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_TOOLTIP) != 0;
+	gtk_widget_set_has_tooltip (GTK_WIDGET (view), has_tooltip);
 }
 
 static void
@@ -955,8 +1025,9 @@ contact_list_view_set_property (GObject      *object,
 static void
 empathy_contact_list_view_class_init (EmpathyContactListViewClass *klass)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
-	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+	GObjectClass     *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
+	GtkTreeViewClass *tree_view_class = GTK_TREE_VIEW_CLASS (klass);
 
 	object_class->finalize = contact_list_view_finalize;
 	object_class->get_property = contact_list_view_get_property;
@@ -967,12 +1038,13 @@ empathy_contact_list_view_class_init (EmpathyContactListViewClass *klass)
 	widget_class->drag_begin         = contact_list_view_drag_begin;
 	widget_class->drag_data_get      = contact_list_view_drag_data_get;
 	widget_class->drag_end           = contact_list_view_drag_end;
-	/* FIXME: noticed but when you drag the row over the treeview
-	 * fast, it seems to stop redrawing itself, if we don't
-	 * connect this signal, all is fine.
-	 */
 	widget_class->drag_motion        = contact_list_view_drag_motion;
 
+	/* We use the class method to let user of this widget to connect to
+	 * the signal and stop emission of the signal so the default handler
+	 * won't be called. */
+	tree_view_class->row_activated = contact_list_view_row_activated;
+
 	signals[DRAG_CONTACT_RECEIVED] =
 		g_signal_new ("drag-contact-received",
 			      G_OBJECT_CLASS_TYPE (klass),
@@ -1025,22 +1097,21 @@ empathy_contact_list_view_init (EmpathyContactListView *view)
 					      NULL, NULL);
 
 	/* Connect to tree view signals rather than override. */
-	g_signal_connect (view,
-			  "button-press-event",
+	g_signal_connect (view, "button-press-event",
 			  G_CALLBACK (contact_list_view_button_press_event_cb),
 			  NULL);
-	g_signal_connect (view,
-			  "row-activated",
-			  G_CALLBACK (contact_list_view_row_activated_cb),
+	g_signal_connect (view, "key-press-event",
+			  G_CALLBACK (contact_list_view_key_press_event_cb),
 			  NULL);
-	g_signal_connect (view,
-			  "row-expanded",
+	g_signal_connect (view, "row-expanded",
 			  G_CALLBACK (contact_list_view_row_expand_or_collapse_cb),
 			  GINT_TO_POINTER (TRUE));
-	g_signal_connect (view,
-			  "row-collapsed",
+	g_signal_connect (view, "row-collapsed",
 			  G_CALLBACK (contact_list_view_row_expand_or_collapse_cb),
 			  GINT_TO_POINTER (FALSE));
+	g_signal_connect (view, "query-tooltip",
+			  G_CALLBACK (contact_list_view_query_tooltip_cb),
+			  NULL);
 }
 
 EmpathyContactListView *
@@ -1124,8 +1195,8 @@ contact_list_view_remove_dialog_show (GtkWindow   *parent,
 	
 	dialog = gtk_dialog_new_with_buttons (window_title, parent,
 					      GTK_DIALOG_MODAL,
+  					      GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
 					      GTK_STOCK_DELETE, GTK_RESPONSE_YES,
-					      GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
 					      NULL);
 	gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
 	 
@@ -1273,10 +1344,9 @@ empathy_contact_list_view_get_contact_menu (EmpathyContactListView *view)
 
 	menu = empathy_contact_menu_new (contact, priv->contact_features);
 
-	if (!menu &&
-	    !(priv->list_features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE)) {
+	if (!(priv->list_features & EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE)) {
 		g_object_unref (contact);
-		return NULL;
+		return menu;
 	}
 
 	if (menu) {
