@@ -28,6 +28,7 @@
 
 #include "empathy-contact.h"
 #include "empathy-tp-contact-list.h"
+#include "empathy-contact-list.h"
 
 G_BEGIN_DECLS
 
@@ -40,10 +41,10 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyContactManager      EmpathyContactManager;
 typedef struct _EmpathyContactManagerClass EmpathyContactManagerClass;
-typedef struct _EmpathyContactManagerPriv  EmpathyContactManagerPriv;
 
 struct _EmpathyContactManager {
-	GObject      parent;
+	GObject parent;
+	gpointer priv;
 };
 
 struct _EmpathyContactManagerClass {
