@@ -40,6 +40,7 @@
 #include <libempathy/empathy-contact-factory.h>
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
+#include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-utils.h>
 
 #include <libempathy-gtk/empathy-images.h>
@@ -286,6 +287,9 @@ chat_window_update (EmpathyChatWindow *window)
 	gint                   page_num;
 	const gchar           *name;
 	guint                  n_chats;
+	GdkPixbuf             *icon;
+	EmpathyContact        *remote_contact;
+	gboolean               avatar_in_icon;
 
 	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
@@ -322,7 +326,18 @@ chat_window_update (EmpathyChatWindow *window)
 		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
 					  EMPATHY_IMAGE_MESSAGE);
 	} else {
-		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
+		empathy_conf_get_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_CHAT_AVATAR_IN_ICON,
+				       &avatar_in_icon);
+
+		if (n_chats == 1 && avatar_in_icon) {
+			remote_contact = empathy_chat_get_remote_contact (priv->current_chat);
+			icon = empathy_pixbuf_avatar_from_contact_scaled (remote_contact, 0, 0);
+			gtk_window_set_icon (GTK_WINDOW (priv->dialog), icon);
+			g_object_unref (icon);
+		} else {
+			gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
+		}
 	}
 }
 
@@ -977,7 +992,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		chat = empathy_chat_window_find_chat (account, strv[1]);
 
 		if (!chat) {
-			empathy_chat_with_contact_id (account, strv[2]);
+			empathy_dispatcher_chat_with_contact_id (account, strv[2]);
 			g_object_unref (account);
 			g_strfreev (strv);
 			return;
@@ -1260,7 +1275,7 @@ empathy_chat_window_get_default (void)
 		chat_window = l->data;
 
 		dialog = empathy_chat_window_get_dialog (chat_window);
-		if (empathy_window_get_is_visible (GTK_WINDOW (GTK_WINDOW (dialog)))) {
+		if (empathy_window_get_is_visible (GTK_WINDOW (dialog))) {
 			/* Found a visible window on this desktop */
 			return chat_window;
 		}
@@ -1298,20 +1313,23 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	/* Reference the chat object */
 	g_object_ref (chat);
 
-	empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
-
-	if (x >= 0 && y >= 0) {
-		/* Let the window manager position it if we don't have
-		 * good x, y coordinates.
-		 */
-		gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
-	}
-
-	if (w > 0 && h > 0) {
-		/* Use the defaults from the glade file if we don't have
-		 * good w, h geometry.
-		 */
-		gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
+	/* If this window has just been created, position it */
+	if (priv->chats == NULL) {
+		empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
+		
+		if (x >= 0 && y >= 0) {
+			/* Let the window manager position it if we don't have
+			 * good x, y coordinates.
+			 */
+			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
+		}
+		
+		if (w > 0 && h > 0) {
+			/* Use the defaults from the glade file if we don't have
+			 * good w, h geometry.
+			 */
+			gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
+		}
 	}
 
 	child = GTK_WIDGET (chat);
